<?php ! defined( 'ABSPATH' ) AND exit( 'Forbidden!' );
/*
	Plugin Name: PuzzleThemes Twitter Feed Widget
	Plugin URI: http://puzzlethemes.net/
	Description: Allows you to easily show your latest tweets in a styled widget.
	Version: 1.0
	Author: PuzzleThemes
	Author URI: http://puzzlethemes.net/
	License: GPLv2
*/


/**
 * Load required dependencies
 */
if ( ! class_exists( 'StormTwitter' ) ) {
	include_once dirname( __FILE__ ) . '/storm-twitter-class/StormTwitter.class.php';
}

if ( ! class_exists( 'Twitter_Autolink' ) ) {
	include_once dirname( __FILE__ ) . '/twitter-text-php/lib/Twitter/Autolink.php';
}


/**
 * Class PuzzleThemes_Twitter_Widget.
 * Displays latest tweets from specified profile.
 * Based on StormTwitter class by Storm Consultancy <https://github.com/stormuk/storm-twitter>
 */
class PuzzleThemes_Twitter_Widget extends WP_Widget {

	/**
	 * PuzzleThemes_Twitter_Widget constructor.
	 */
	public function __construct() {
		// load textdomain first
		$lng_dir = plugin_basename( dirname( __FILE__ ) ) . '/languages';
		load_plugin_textdomain( 'pzt-twitter-widget', false, $lng_dir );

		// then call parent constructor
		parent::__construct(
			// Base ID of your widget
			'pzt_twitter_widget',

			// Widget name will appear in UI
			esc_attr__('Andaman: Tweets Timeline', 'pzt-twitter-widget'),

			// Widget description
			array(
				'description' => esc_attr__( 'Displays latest tweets from specified profile.', 'pzt-twitter-widget' )
			)
		);
	}


	/**
	 * Generates markup output by widget
	 *
	 * @param array $args
	 * @param array $instance
	 */
	public function widget( $args, $instance ) {
		$instance = $this->parse_instance( $instance );
		$title = apply_filters( 'widget_title', $instance['title'] );

		if ( ! function_exists( 'curl_init' ) ) {
			trigger_error( 'PuzzleThemes Twitter Feed widget requires CURL PHP which is not installed on your server. Please install/enable it and try again.' );

			return;
		}

		// setup stormTwitter config
		$upload_dir = wp_get_upload_dir();
		$twitter    = new StormTwitter( array(
			'directory'    => $upload_dir['basedir'] . DIRECTORY_SEPARATOR,
			'key'          => $instance['twitter_app_key'],
			'secret'       => $instance['twitter_app_secret'],
			'token'        => $instance['twitter_token'],
			'token_secret' => $instance['twitter_token_secret'],
			'cache_expire' => 3600
		) );

		// fetch tweets
		$tweets = $twitter->getTweets(
            $instance['username'],
            $instance['tweets_count'],
            array( 'exclude_replies' => true )
        );

		// error handling
		if ( isset( $tweets['error'] ) ) {
			trigger_error( $tweets['error'] );
			return;
		}

		// autolink all tweets
		foreach ( $tweets as $k => $tweet ) {
			$tweets[ $k ]['text'] = Twitter_Autolink::create( $tweet['text'], false )->addLinks();
		}

		// before and after widget arguments are defined by themes
		echo $args['before_widget'];
		if ( ! empty( $title ) ) echo $args['before_title'] . $title . $args['after_title'];

		// render widget
		$default_template = dirname( __FILE__ ) . '/widget-template.php';
		$widget_template  = apply_filters( 'pzt/twitter_widget/template_file', $default_template );
		$this->render_template( $widget_template, array( 'tweets' => $tweets ) );

		// after argument
		echo $args['after_widget'];
	}


	/**
	 * Widget Backend
	 *
	 * @param array $instance
	 *
	 * @return string|void
	 */
	public function form( $instance ) {
		$instance = $this->parse_instance( $instance );

		// widget title
		$this->render_field( 'title', esc_html__( 'Title:' ), $instance['title'] );

		// twitter api key
		echo '<p style="font-style:italic;border-left:3px solid #eee;padding-left:15px;">';
		_e('You should obtain these values from your own application at <a href="https://apps.twitter.com/" target="_blank">Twitter Apps page</a>.', 'pzt-twitter-widget');
		echo '</p>';

		$this->render_field( 'twitter_app_key', esc_html__( 'Twitter API key:' ), $instance['twitter_app_key'] );

		// twitter api secret
		$this->render_field( 'twitter_app_secret', esc_html__( 'Twitter API secret:' ), $instance['twitter_app_secret'] );

		// twitter token
		$this->render_field( 'twitter_token', esc_html__( 'Twitter access token:' ), $instance['twitter_token'] );

		// twitter token secret
		$this->render_field( 'twitter_token_secret', esc_html__( 'Twitter access token secret:' ), $instance['twitter_token_secret'] );

		echo '<hr/>';

		// tweets count
		$this->render_field( 'tweets_count', esc_html__( 'Tweets to display:' ), $instance['tweets_count'] );

		// twitter username
		$this->render_field( 'username', esc_html__( 'Twitter username:' ), $instance['username'] );
	}


	/**
	 * Updating widget replacing old instances with new
	 *
	 * @param array $new_instance
	 * @param array $old_instance
	 *
	 * @return array
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = $this->parse_instance( $old_instance );
		$instance = array_merge( $instance, $new_instance );

		return $instance;
	}


	/**
	 * Parse instance and fill default values
	 *
	 * @param $instance
	 *
	 * @return array
	 */
	private function parse_instance( $instance ) {
		return wp_parse_args(
			$instance,
			array(
				'title'                => esc_html__( 'Twitter Feed', 'pzt-twitter-widget' ),
				'twitter_app_key'      => '',
				'twitter_app_secret'   => '',
				'twitter_token'        => '',
				'twitter_token_secret' => '',
				'tweets_count'         => 2,
				'username'             => ''
			)
		);
	}


	/**
	 * Renders field for the
	 *
	 * @param $field_id
	 * @param $field_label
	 * @param $field_value
	 */
	private function render_field( $field_id, $field_label, $field_value, $field_description = null ) { ?>
		<p>
			<label for="<?php echo $this->get_field_id( $field_id ); ?>">
				<?php echo $field_label; ?>
			</label>

			<input class="widefat" id="<?php echo $this->get_field_id( $field_id ); ?>" name="<?php echo $this->get_field_name( $field_id ); ?>" type="text" value="<?php echo esc_attr( $field_value ); ?>" />

			<?php if ( $field_description ): ?>
				<small><?php echo $field_description; ?></small>
			<?php endif; ?>
		</p>
	<?php }


	/**
	 * Renders given template with provided variables
	 *
	 * @param $template_file
	 * @param $atts
	 */
	private function render_template( $template_file, $atts ) {
		extract( $atts );
		include $template_file;
	}
}


/**
 * Register widget on @widgets_init hook
 */
add_action( 'widgets_init', 'pzt_themes_register_twitter_widget' );
function pzt_themes_register_twitter_widget() {
	register_widget( 'PuzzleThemes_Twitter_Widget' );
}